package com.xdf.creative.support.aop;

import com.alibaba.fastjson.JSON;
import com.xdf.creative.module.entity.SysOperationLog;
import com.xdf.creative.module.service.SysOperationLogService;
import com.xdf.creative.util.IpUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Date;

/**
 * @author : lgw
 * @date : 17:57 2020/3/2
 */
@Aspect
@Component
public class SysOperationLogAspect {
    @Autowired
    private SysOperationLogService sysOperationLogService;
    private static final Logger logger = LoggerFactory.getLogger(SysOperationLogAspect.class);
    //定义切点 @Pointcut
    //在注解的位置切入代码
    @Pointcut("@annotation( com.xdf.creative.support.aop.OperationLogger)")
    public void logPoinCut() {

    }

    //切面 配置通知
    @AfterReturning("logPoinCut()")
    public void saveSysLog(JoinPoint joinPoint) {
        System.out.println("切面。。。。。");
        //保存日志
        SysOperationLog sysOperationLog = new SysOperationLog();

        //从切面织入点处通过反射机制获取织入点处的方法
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        //获取切入点所在的方法
        Method method = signature.getMethod();

        //获取操作
        OperationLogger operationLogger = method.getAnnotation(OperationLogger.class);
        if (operationLogger != null) {
            String value = operationLogger.value();
            sysOperationLog.setDetail(value);//保存获取的操作
        }
        //获取请求的类名
        String className = joinPoint.getTarget().getClass().getName();
        //获取请求的方法名
        String methodName = method.getName();
        sysOperationLog.setMethod(className + "." + methodName);

        //请求的参数
        Object[] args = joinPoint.getArgs();
        //将参数所在的数组转换成json
//        String params = JSON.toJSONString(args);
//        sysOperationLog.setParams(params);

        sysOperationLog.setCreateTime(new Date());
        //获取用户名
//        sysOperationLog.setUserId(ShiroUtils.getUserEntity().getUsername());
        sysOperationLog.setUserId((long)1);
        //获取用户ip地址
        sysOperationLog.setApi(IpUtil.getRequestIp());
        //调用service保存SysLog实体类到数据库
        sysOperationLogService.save(sysOperationLog);
    }

}
